using System;
using Microsoft.Xna.Framework;
using Terraria;
using Terraria.DataStructures;
using Terraria.Enums;
using Terraria.ID;
using Terraria.ModLoader;
using Terraria.ObjectData;

namespace ExampleMod.Tiles {
public class ExampleChest : ModTile
{
    public override void SetDefaults()
    {
        Main.tileSpelunker[Type] = true;
        Main.tileContainer[Type] = true;
        Main.tileShine2[Type] = true;
        Main.tileShine[Type] = 1200;
        Main.tileFrameImportant[Type] = true;
        Main.tileNoAttach[Type] = true;
        Main.tileValue[Type] = 500;
        TileObjectData.newTile.CopyFrom(TileObjectData.Style2x2);
        TileObjectData.newTile.Origin = new Point16(0, 1);
        TileObjectData.newTile.CoordinateHeights = new int[]{16, 18};
        TileObjectData.newTile.HookCheck = new PlacementHook(new Func<int, int, int, int, int, int>(Chest.FindEmptyChest), -1, 0, true);
        TileObjectData.newTile.HookPostPlaceMyPlayer = new PlacementHook(new Func<int, int, int, int, int, int>(Chest.AfterPlacement_Hook), -1, 0, false);
        TileObjectData.newTile.AnchorInvalidTiles = new int[]{127};
        TileObjectData.newTile.StyleHorizontal = true;
        TileObjectData.newTile.LavaDeath = false;
        TileObjectData.newTile.AnchorBottom = new AnchorData(AnchorType.SolidTile | AnchorType.SolidWithTop | AnchorType.SolidSide, TileObjectData.newTile.Width, 0);
        TileObjectData.addTile(Type);
        mapColor = new Color(200, 200, 200);
        mapName = "Chest";
        adjTiles = new int[]{TileID.Containers};
        chest = "Example Chest";
    }

    public override void NumDust(int i, int j, bool fail, ref int num)
    {
        num = 1;
    }

    public override bool CreateDust(int i, int j, ref int type)
    {
        ModDust.NewDust(new Vector2(i, j) * 16f, 16, 16, mod, "Sparkle");
        return false;
    }

    public override bool CanKillTile(int i, int j, ref bool blockDamaged)
    {
        Tile tile = Main.tile[i, j];
        int left = i;
        int top = j;
        if(tile.frameX % 36 != 0)
        {
            left--;
        }
        if(tile.frameY != 0)
        {
            top--;
        }
        return Chest.CanDestroyChest(left, top);
    }

    public override void KillMultiTile(int i, int j, int frameX, int frameY)
    {
        Item.NewItem(i * 16, j * 16, 32, 32, mod.ItemType("ExampleChest"));
        Chest.DestroyChest(i, j);
    }

    public override void RightClick(int i, int j)
    {
        Player player = Main.player[Main.myPlayer];
        Tile tile = Main.tile[i, j];
        Main.mouseRightRelease = false;
        int left = i;
        int top = j;
        if(tile.frameX % 36 != 0)
        {
            left--;
        }
        if(tile.frameY != 0)
        {
            top--;
        }
        if(player.sign >= 0)
        {
            Main.PlaySound(11, -1, -1, 1);
            player.sign = -1;
            Main.editSign = false;
            Main.npcChatText = "";
        }
        if(Main.editChest)
        {
            Main.PlaySound(12, -1, -1, 1);
            Main.editChest = false;
            Main.npcChatText = "";
        }
        if(player.editedChestName)
        {
            NetMessage.SendData(33, -1, -1, Main.chest[player.chest].name, player.chest, 1f, 0f, 0f, 0, 0, 0);
            player.editedChestName = false;
        }
        if(Main.netMode == 1)
        {
            if(left == player.chestX && top == player.chestY && player.chest >= 0)
            {
                player.chest = -1;
                Recipe.FindRecipes();
                Main.PlaySound(11, -1, -1, 1);
            }
            else
            {
                NetMessage.SendData(31, -1, -1, "", left, (float)top, 0f, 0f, 0, 0, 0);
                Main.stackSplit = 600;
            }
        }
        else
        {
            int chest = Chest.FindChest(left, top);
            if(chest >= 0)
            {
                Main.stackSplit = 600;
                if(chest == player.chest)
                {
                    player.chest = -1;
                    Main.PlaySound(11, -1, -1, 1);
                }
                else
                {
                    player.chest = chest;
                    Main.playerInventory = true;
                    Main.recBigList = false;
                    player.chestX = left;
                    player.chestY = top;
                    Main.PlaySound(player.chest < 0 ? 10 : 12, -1, -1, 1);
                }
                Recipe.FindRecipes();
            }
        }
    }

    public override void MouseOver(int i, int j)
    {
        Player player = Main.player[Main.myPlayer];
        Tile tile = Main.tile[i, j];
        int left = i;
        int top = j;
        if(tile.frameX % 36 != 0)
        {
            left--;
        }
        if(tile.frameY != 0)
        {
            top--;
        }
        int chest = Chest.FindChest(left, top);
        player.showItemIcon2 = -1;
        if(chest < 0)
        {
            player.showItemIconText = Lang.chestType[0];
        }
        else
        {
            player.showItemIconText = Main.chest[chest].name.Length > 0 ? Main.chest[chest].name : "Example Chest";
            if(player.showItemIconText == "Example Chest")
            {
                player.showItemIcon2 = mod.ItemType("ExampleChest");
                player.showItemIconText = "";
            }
        }
        player.noThrow = 2;
        player.showItemIcon = true;
    }
}}